@@ -1,11 +1,16 @@
 import arcade
-### default
+### default win size
 
-WIDTH = 500
-HEIGHT = 800
+WIN_WIDTH = 500
+WIN_HEIGHT = 800
+
+###
+
+WIN_MIN_WIDTH = 300
+WIN_MIN_HEIGHT = 300
+WIN_MAX_WIDTH = 1600
+WIN_MAX_HEIGHT = 1000
 
-MIN_WIDTH = 200
-MIN_HEIGHT = 200
 
 TITLE = '物理碰撞模擬'
 
@@ -29,16 +34,17 @@ ELASTICITY = 0.95
 
 VELOCITY_LIMIT = 1000
 
-POSITION_Y_MAX = HEIGHT * 4
-POSITION_Y_MIN = HEIGHT * -1
-POSITION_X_MAX = WIDTH * 2
-POSITION_X_MIN = WIDTH * -1
+
+RECYCLE_Y_MAX_MUL = 3
+RECYCLE_Y_MIN_MUL = -1
+RECYCLE_X_MAX_MUL = 2
+RECYCLE_X_MIN_MUL = -1
 
 SEG_THICKNESS = 6
 
 
 #assist
-ASSIST_MARK_PERIOD = 5
+ASSIST_MARK_PERIOD = 0.5
 
 
 #shape
@@ -49,10 +55,10 @@ SHAPE_COLORS = (arcade.color.WINDSOR_TAN,
                arcade.color.AO,
                )
 
-CIRCLE_RADIUS_MAX = 30
-CIRCLE_RADIUS_MIN = 10
+CIRCLE_RADIUS_MAX = 40
+CIRCLE_RADIUS_MIN = 15
 
-BOX_WIDTH_MAX = 60
+BOX_WIDTH_MAX = 80
 BOX_WIDTH_MIN = 20
 
 BOX_ELASTICITY = 0.5
